listo.class('ListoCustomIcons', function (listo, $) {
	const { ajax, hooks, template, data: config } = listo;

	return class {
		constructor(container, opts = {}) {
			this.id = opts.id;
			this.smart = true;

			this.$container = $(container);
			this.$toggle = this.$container.find('[data-listo=custom-icon-toggle]');
			this.$disabler = this.$container.find('[data-listo=custom-icon-disabler]').hide();
			this.$popup = this.$container.find('[data-listo=custom-icon-popup]');
			this.$search = this.$popup.find('[type=text]');
			this.$list = this.$popup.find('[data-listo=custom-icon-list]');
			this.$loading = this.$popup.find('[data-listo=custom-icon-loading]').hide();
			this.$clear = this.$popup.find('[data-listo=custom-icon-clear]').hide();
			this.$remove = this.$popup.find('[data-listo=custom-icon-remove]').hide();

			this.tmplItem = this.$container.find('[data-listo=tmpl-item]').text();

			this.$toggle.on('click', e => this.toggle(e));
			this.$popup.on('click', e => e.stopPropagation());
			this.$search.on('input', () => this.search());
			this.$clear.on('click', () => this.clear());
			this.$list.on('click', '[data-option]', e => this.onSelectIcon(e));
			this.$remove.on('click', e => this.onRemoveIcon(e));
			$(document).on('click', () => this.$popup.hide());

			hooks.addAction('set_current_collection', data => this.update(data));
			hooks.addAction('collection_updated', data => this.update(data));
		}

		update(data) {
			this.smart = +data.smart;
			this.$toggle.attr('class', `lo-list__icon ${data.icon || ''}`);
			this.$toggle.find('[data-field=initials]').html(data.initials || '');
			this.$popup.hide();
			this.$list.empty();
			this.$search.val(data.title);
			data.icon ? this.$remove.show() : this.$remove.hide();
		}

		toggle(e) {
			e.stopPropagation();

			if (this.smart || this.$popup.is(':visible')) {
				this.$popup.hide();
				this.$list.empty();
				return;
			}

			this.search();
			this.$popup.show();
			this.$search.select();
		}

		search() {
			this.$clear.hide();
			this.$loading.show();
			this.$list.addClass('is-loading');

			clearTimeout(this.searchTimer);
			let timer = (this.searchTimer = setTimeout(() => {
				let keyword = this.$search.val().trim();
				return ajax('listowp/v1/icons', { s: keyword, limit: 105 }).done(json => {
					if (timer === this.searchTimer) {
						this.$loading.hide();
						this.$list.removeClass('is-loading');
						keyword ? this.$clear.show() : this.$clear.hide();
						this.render(json.icons);
					}
				});
			}, 1000));
		}

		render(icons) {
			let html = '';
			for (const key in icons) {
				html += template(this.tmplItem)({ icon: key });
			}

			this.$list.html(html);
		}

		clear() {
			this.$clear.hide();
			this.$search.val('');
			this.search();
		}

		onSelectIcon(e) {
			e.preventDefault();
			e.stopPropagation();

			this.$popup.hide();
			this.$remove.show();
			this.$disabler.show();

			let id = hooks.applyFilters('current_collection_id');
			let icon = $(e.currentTarget).data('option');
			let params = { id, icon: `fa-solid fa-${icon}` };

			// Pre-update icon.
			let classList = this.$toggle.get(0).classList;
			let iconClasses = [...classList].filter(i => i.match(/^fa-/));
			classList.remove(...iconClasses);
			classList.add(...params.icon.split(' '));

			return ajax('listowp/v1/collections', params, 'PATCH')
				.fail(() => {
					this.$disabler.hide();
					classList.remove(...params.icon.split(' '));
					classList.add(...iconClasses);
				})
				.done(() => {
					ajax('listowp/v1/collections', { id })
						.always(() => this.$disabler.hide())
						.done(json => {
							if (json instanceof Array && json[0]) {
								hooks.doAction('collection_updated', json[0]);
							}
						});
				});
		}

		onRemoveIcon(e) {
			e.preventDefault();
			e.stopPropagation();

			this.$popup.hide();
			this.$remove.hide();
			this.$disabler.show();

			let id = hooks.applyFilters('current_collection_id');
			let params = { id, icon: '' };

			// Pre-update icon.
			let classList = this.$toggle.get(0).classList;
			let iconClasses = [...classList].filter(i => i.match(/^fa-/));
			classList.remove(...iconClasses);

			return ajax('listowp/v1/collections', params, 'PATCH')
				.fail(() => {
					this.$disabler.hide();
					classList.add(...iconClasses);
				})
				.done(() => {
					ajax('listowp/v1/collections', { id })
						.always(() => this.$disabler.hide())
						.done(json => {
							if (json instanceof Array && json[0]) {
								hooks.doAction('collection_updated', json[0]);
							}
						});
				});
		}
	};
});
