listo.class('ListoItemDue', function (listo, $) {
	const { ajax, hooks, template, data: config } = listo;
	const { h12to24, h24to12, h24ampm, pad } = listo.string;

	return class extends listo.class('ListoAbstract') {
		constructor(container) {
			super(container);

			$(document).on('click', () => {
				this.$container.find('[data-listo=item]').each((i, el) => {
					this.hide(el);
				});
			});
		}

		update(e, data) {
			let $item = this.$item(e);
			let id = $item.data('id');

			data = Object.assign({}, data, { id });

			return ajax('listowp/v1/items', data, 'PATCH').done(json => {
				if (json instanceof Array && json[0]) {
					let dueTmpl = template(config('templates.item_due'));
					let item = hooks.applyFilters('item_map', json[0]);

					this.$toggle(e).replaceWith(dueTmpl(item));
				}

				hooks.doAction('maintenance');
			});
		}

		$item(e) {
			return $(e.currentTarget || e).closest('[data-listo=item]');
		}

		$toggle(e) {
			return this.$item(e).find('[data-listo=item-due]');
		}

		$popup(e) {
			return this.$item(e).find('[data-listo=item-due-popup]');
		}

		show(e) {
			let $popup = this.$popup(e);
			let $popups = this.$container.find('[data-listo=item-due-popup]');

			// Hide other popups.
			$popups.not($popup).filter(':visible').hide();
			$popup.show();

			// Default to today if date is empty.
			let $date = $popup.find('[data-listo=item-due-date]');
			if (!$date.val()) {
				let now = new Date();
				let year = now.getFullYear();
				let month = pad(now.getMonth() + 1, 2, '0');
				let date = pad(now.getDate(), 2, '0');

				$date.val(`${year}-${month}-${date}`).trigger('change');
			}

			hooks.doAction('item_due_show', $popup);

			let canSubmit = hooks.applyFilters('item_due_can_submit', true, $popup);
			this.toggleSubmit($popup, canSubmit);
		}

		hide(e) {
			this.$popup(e).hide();
		}

		toggle(e) {
			this.$popup(e).is(':visible') ? this.hide(e) : this.show(e);
		}

		/**
		 * Decide if the Submit button should be enabled.
		 *
		 * @param {jQuery} $popup
		 * @param {boolean|jQuery.Deferred} canSubmit
		 */
		toggleSubmit($popup, canSubmit) {
			let $submit = $popup.find('[data-listo=item-due-set]');

			if ('boolean' === typeof canSubmit) {
				canSubmit ? $submit.removeAttr('disabled') : $submit.attr('disabled', 'disabled');
			} else if (canSubmit && canSubmit.done) {
				$submit.attr('disabled', 'disabled');
				canSubmit.done(canSubmit => {
					canSubmit
						? $submit.removeAttr('disabled')
						: $submit.attr('disabled', 'disabled');
				});
			}
		}

		onItemClick(e) {
			this.hide(e);
		}

		onItemDueClick(e) {
			e.stopPropagation();

			this.toggle(e);
		}

		onItemDuePopupClick(e) {
			e.stopPropagation();
		}

		onItemDueDateClick(e) {
			e.currentTarget.showPicker();
		}

		onItemDueDateChange(e) {
			let $date = $(e.currentTarget);
			let $mirror = $date.siblings('[type=text]');
			let $loading = $date.siblings('[data-listo=loading]');
			let date = $date.val();

			if (!date) {
				$mirror.val('').css('opacity', '');
				$loading.hide();
				return;
			}

			$mirror.css('opacity', 0);
			$loading.show();

			ajax('listowp/v1/time', { date }).done(json => {
				if (json.date_formatted) {
					$loading.hide();
					$mirror.val(json.date_formatted).css('opacity', '');
				}
			});

			hooks.doAction('item_due_date_changed', this.$popup(e));
		}

		onItemDueHoursChange(e) {
			let $item = this.$item(e);
			let $hours12h = $item.find('[data-listo=item-due-hours-12h]');
			let $ampm = $item.find('[data-listo=item-due-ampm]');
			let hours = +e.currentTarget.value;

			$hours12h.val(h24to12(hours));
			$ampm.val(h24ampm(hours));
		}

		onItemDueHours12hChange(e) {
			let $item = this.$item(e);
			let $ampm = $item.find('[data-listo=item-due-ampm]');
			let $hours = $item.find('[data-listo=item-due-hours]');

			$hours.val(h12to24(+e.currentTarget.value, $ampm.val()));
		}

		onItemDueMinutesChange(e) {
			this.$item(e).find('[data-listo=item-due-minutes-12h]').val(e.currentTarget.value);
		}

		onItemDueMinutes12hChange(e) {
			this.$item(e).find('[data-listo=item-due-minutes]').val(e.currentTarget.value);
		}

		onItemDueAmpmChange(e) {
			let $item = this.$item(e);
			let $hours12h = $item.find('[data-listo=item-due-hours-12h]');
			let $hours = $item.find('[data-listo=item-due-hours]');

			$hours.val(h12to24($hours12h.val(), e.currentTarget.value));
		}

		onItemDueSetClick(e) {
			e.stopPropagation();

			let $popup = this.$popup(e);

			let date = $popup.find('[data-listo=item-due-date]').val();
			let hours = $popup.find('[data-listo=item-due-hours]').val();
			let minutes = $popup.find('[data-listo=item-due-minutes]').val();
			let time = `${hours}:${minutes}:00`;

			let data = { due: `${date}${date ? ' ' + time : ''}` };
			data = hooks.applyFilters('item_due_submit_data', data, $popup);

			this.update(e, data);
		}

		onItemDueClearClick(e) {
			e.stopPropagation();

			this.update(e, { due: '' });
		}
	};
});
