listo.class('ListoPreferences', function (listo, $) {
	const { ajax, hooks, template, data: config } = listo;

	const ANIM_SPEED = +config('animation_speed');

	return class extends listo.class('ListoAbstract') {
		constructor(container) {
			super(container);

			this.$itemsOrder = this.$container.find('[data-listo=items-order]');
			this.$itemsOrderDir = this.$container.find('[data-listo=items-order-dir]');
			this.$itemsOrderDirBox = this.$container.find('[data-listo=items-order-dir-box]');
			this.$itemsHideDone = this.$container.find('[data-listo=items-hide-done]');
			this.$timezone = this.$container.find('[data-listo=timezone]');

			this.getPreferences().done(json => {
				hooks.doAction('preferences_init', json);
			});

			hooks.addAction('preferences_toggle', () => this.onPreferencesToggle());
			hooks.addAction('toggle_completed', () => this.onToggleCompleted());
		}

		getPreferences() {
			return ajax('listowp/v1/preferences').done(json => {
				if ('undefined' !== typeof json.items_order) {
					this.$itemsOrder.val(json.items_order);
					if (json.items_order && 'undefined' !== typeof json.items_order_dir) {
						this.$itemsOrderDirBox.show();
						this.$itemsOrderDir.val(json.items_order_dir);
					} else {
						this.$itemsOrderDirBox.hide();
					}
				}

				if ('undefined' !== typeof json.items_hide_done) {
					this.$itemsHideDone.val(json.items_hide_done);
				}

				if ('undefined' !== typeof json.timezone) {
					this.$timezone.val(json.timezone);
				}
			});
		}

		setPreferences(id, value) {
			return ajax('listowp/v1/preferences', { id, value }, 'PATCH');
		}

		onPreferencesToggle() {
			this.$container.fadeIn(ANIM_SPEED);
			$(document).on('keydown.listo-preferences', e => {
				if ('Escape' === e.code) {
					$(document).off('keydown.listo-preferences');
					this.$container.fadeOut(ANIM_SPEED);
				}
			});
		}

		onToggleCompleted() {
			let select = this.$itemsHideDone.get(0);

			if (!select.disabled) {
				select.selectedIndex = select.selectedIndex ? 0 : 1;
				select.disabled = true;

				let value = select.value;

				this.setPreferences('items_hide_done', value)
					.always(() => (select.disabled = false))
					.done(() => hooks.doAction('preferences_updated', { items_hide_done: value }));
			}
		}

		onPreferencesCloseClick(e) {
			$(document).off('keydown.listo-preferences');
			this.$container.fadeOut(ANIM_SPEED);
		}

		onItemsOrderChange() {
			let $select = this.$itemsOrder;
			let value = $select.val() || '';
			if (!value) {
				this.$itemsOrderDir[0].selectedIndex = 0;
			}

			this.progress($select, 'loading');
			this.setPreferences('items_order', value)
				.fail(() => this.progress($select, 'fail'))
				.done(() => {
					this.progress($select, 'done');
					if (value) {
						this.$itemsOrderDirBox.show();
						this.$itemsOrderDir.trigger('change');
					} else {
						this.$itemsOrderDirBox.hide();
						hooks.doAction('preferences_updated', {
							items_order: value,
							items_order_dir: this.$itemsOrderDir.val()
						});
					}
				});
		}

		onItemsOrderDirChange() {
			let $select = this.$itemsOrderDir;
			let value = $select.val();

			this.progress($select, 'loading');
			this.setPreferences('items_order_dir', value)
				.fail(() => this.progress($select, 'fail'))
				.done(() => {
					this.progress($select, 'done');
					hooks.doAction('preferences_updated', {
						items_order: this.$itemsOrder.val(),
						items_order_dir: value
					});
				});
		}

		onItemsHideDoneChange() {
			let $select = this.$itemsHideDone;
			let value = $select.val();

			this.progress($select, 'loading');
			this.setPreferences('items_hide_done', value)
				.fail(() => this.progress($select, 'fail'))
				.done(() => {
					this.progress($select, 'done');
					hooks.doAction('preferences_updated', { items_hide_done: value });
				});
		}

		onTimezoneChange() {
			let $select = this.$timezone;
			let value = $select.val();

			this.progress($select, 'loading');
			this.setPreferences('timezone', value)
				.fail(() => this.progress($select, 'fail'))
				.done(() => {
					this.progress($select, 'done');
					hooks.doAction('preferences_updated', { timezone: value });
				});
		}

		onGdprDeleteClick(e) {
			e.preventDefault();
			e.stopPropagation();

			if (!confirm(config('lang.gdpr_delete_confirmation'))) {
				return;
			}

			return ajax('listowp/v1/gdpr', {}, 'DELETE').done(json => {
				this.$itemsOrder[0].selectedIndex = 0;
				this.$itemsOrderDir[0].selectedIndex = 0;
				this.$itemsOrderDirBox.hide();
				this.$itemsHideDone[0].selectedIndex = 0;
				this.$timezone.val('+00:00');

				hooks.doAction('gdpr_deleted');
			});
		}
	};
});
