<?php

class Listo_Icons {

	private static $instance;
	public $icons;
	public static function get_instance()
	{
		return isset(self::$instance) ? self::$instance : self::$instance = new self;
	}

	private function __construct() {
		// Load icon definitions
		$json = file_get_contents(Listo_Frontend_Assets::get_instance()->assets_dir.'icons/fa/icons.json');
		$this->icons = json_decode($json,true);
	}

	public function search($keywords, $limit = 10000, $allow_empty_result = FALSE) {

		// Get and clean up search string
		$keywords = strtolower($keywords);
		$keywords = preg_replace("/[^a-z0-9 ]/", "", $keywords);

		$search  = $keywords;

		// Make sure each search term is a separate word
		if(strstr($keywords,' ')) {
			$keywords = explode( ' ', $keywords );
		} else {
			$keywords = [ $keywords ];
		}

		// Result counters
		$results_total = 0;
		$results_shown = 0;

		// Result array with meta
		$result = [
			'meta' => [
				'search'=>$search,
				'user'=>Listo_User::get_id(),
				'limit' => $limit,
			],
			'icons' => [],
			'accuracy' => [],
		];

		// Loop through icons and search the terms
		$total_loops = 0;
		foreach($this->icons as $fa=>$icon) {

			// Even if search string is empty we will show $limit first icons
			$found = strlen($search) ? 0 : 1;

			// Search
			if(strlen($search)) {

				// Add the actual fa key to the terms (improves accuracy)
				$icon['search'][]=$fa;

				foreach($icon['search'] as $keyword) {
					$icon['search'][] =$keyword.'s';
				}

				// Execute search
				foreach ( $icon['search'] as $term ) {
					foreach($keywords as $keyword) {
						if ( stristr( $term, $keyword ) ) {

							// Exact match
							if($term == $keyword) {
								@$result['accuracy'][ $fa ] += 1000;
							}

							// Similarity score (0-1000)
							$perc = 0;
							$sim = similar_text($term, $keyword, $perc);
							@$result['accuracy'][ $fa ] += (int) floor($perc)*10;

							// Any other match
							@$result['accuracy'][ $fa ] += 10;

							// The further we are in the file, the least popular the icon should be
							//@$result['accuracy'][$fa]  -= ++$total_loops;

							$found = 1;
						}
					}
				}
			}

			// No match, do not increment found count
			if(!$found) continue;
			@$result['accuracy'][ $fa ] ++;
			$results_total++;
		}

		// Sort results by accuracy
		if(strlen($search)) {

			uasort( $result['accuracy'], function ( $a, $b ) {
				if ( $a == $b ) {
					return 0;
				}

				return ( $a > $b ) ? - 1 : 1;
			} );
		}

		if ( count( $result['accuracy'] ) ) {

			foreach ( $result['accuracy'] as $fa => $score ) {

				if ( $results_shown >= $limit ) {
					break;
				}

				$icon             = $this->icons[ $fa ];
				$icon['accuracy'] = $score;
				unset( $icon['search'] );
				$result['icons'][ $fa ] = $icon;
				$results_shown ++;
			}
		}


		// Finalize meta
		unset($result['accuracy']);
		$result['meta']['results_shown'] = $results_shown;
		$result['meta']['results_total'] = $results_total;

		if(!count($result['icons'])) {
			if($allow_empty_result) {
				return $result;
			} else {
				$result = $this->search( 'todo', $limit );
			}

		}

		return $result;
	}

	public function pick_best($keywords) {

		$result = $this->search($keywords,1, TRUE);

		if(count($result['icons'])) {
			return array_keys($result['icons'])[0];
		}

		return FALSE;
	}

}